//
// Created by bnozka on 29.05.2021.
//

#include <libwebsockets.h>

#include <utility>
#include "Disc.h"

Disc::Disc(std::filesystem::path path, std::string name, std::string model, std::string wwid, unsigned long size):
path{std::move(path)},
name{std::move(name)},
model{std::move(model)},
wwid{std::move(wwid)},
dev_size{size},
buffer_size_checker(this->path, buffer_size),
zero_writer(this->path, buffer_size, dev_size),
thread{&Disc::run, this}
{
}

Disc::~Disc()
{
    run_thread = false;
    thread.join();
}

void Disc::run()
{
    TickableState* current_state_executor = nullptr;
    while (run_thread)
    {
        switch (state)
        {
        case ERROR:
            if(current_state_executor !=nullptr)
            {
                current_state_executor = nullptr;
            }
        case IDLE:
            std::this_thread::sleep_for(std::chrono::milliseconds(100));
            break;
        case CHECKING_HEALTH:
            state = CHECKING_BUFFER_SIZE;
            break;
        case CHECKING_BUFFER_SIZE:
            if(current_state_executor != &buffer_size_checker)
            {
                current_state_executor = &buffer_size_checker;
                buffer_size_checker.reset();
            }
            buffer_size_checker.tick();
            if(buffer_size_checker.getState() == TickableState::FINISH)
            {
                current_state_executor = nullptr;
                state = ZEROING;
            }
            break;
        case ZEROING:
            if(current_state_executor != &zero_writer)
            {
                current_state_executor = &zero_writer;
                zero_writer.reset();
            }
            zero_writer.tick();
            if(zero_writer.getState() == TickableState::FINISH)
            {
                current_state_executor = nullptr;
                state = DONE_PING;
            }
            break;
        case DONE_PING:
            if(current_state_executor != &beeper)
            {
                current_state_executor = &beeper;
            }
            beeper.tick();
            break;
        case ABORTING:
            if(current_state_executor !=nullptr)
            {
                current_state_executor->abort();
                current_state_executor->reset();
            }
            state = IDLE;
            break;
        case END_OF_CONTROL_STATES:break;
        }

        if(current_state_executor != nullptr &&
           current_state_executor->getState() == TickableState::ERROR)
        {
            state = ERROR;
            last_error = current_state_executor->getLastError();
        }
    }

    if(current_state_executor !=nullptr)
    {
        current_state_executor->abort();
    }
}

void Disc::startZeroing()
{
    if(state == IDLE || state == ERROR)
    {
        state = CHECKING_HEALTH;
    }
}

void Disc::abort()
{
    if (state > END_OF_CONTROL_STATES)
    {
        state = ABORTING;
    }
}