//
// Created by bnozka on 29.05.2021.
//

#ifndef DWIPE_DISC_H
#define DWIPE_DISC_H

#include <thread>
#include <filesystem>
#include <array>

#include "states/BufferSizeChecker.h"
#include "states/HealthChecker.h"
#include "states/ZeroWriter.h"
#include "states/Beep.h"

class Disc {
public:
    enum State {
        IDLE,
        ERROR,
        ABORTING,
        END_OF_CONTROL_STATES,
        CHECKING_HEALTH,
        CHECKING_BUFFER_SIZE,
        ZEROING,
        DONE_PING,
    };

    constexpr static std::array<const char* const, 8> state_strings {
            "Idle",
            "Error",
            "Aborting",
            "",
            "Checking health",
            "Checking buffer size",
            "Zeroing",
            "Done",
    };

    Disc(std::filesystem::path path, std::string name, std::string model, std::string wwid, unsigned long size);
    ~Disc();

    void startZeroing();
    void abort();

    [[nodiscard]] const std::filesystem::path& getPath() const { return path; };
    [[nodiscard]] const std::string& getName() const { return name; };
    [[nodiscard]] const std::string& getModel() const { return model; };
    [[nodiscard]] const std::string& getWwid() const { return wwid; };
    [[nodiscard]] unsigned long getSize() const { return dev_size; };
    [[nodiscard]] unsigned int getBufferSize() const { return buffer_size; };
    [[nodiscard]] int getLastError() const { return last_error; };
    [[nodiscard]] State getState() const { return state; };

    [[nodiscard]] const BufferSizeChecker& getBufferSizeChecker() const { return buffer_size_checker; };
    [[nodiscard]] const ZeroWriter& getZeroWriter() const { return zero_writer; };
private:
    void run();

    State state{IDLE};
    bool run_thread{true};
    const std::filesystem::path path;
    const std::string name;
    const std::string model;
    const std::string wwid;
    unsigned long dev_size;
    unsigned int buffer_size{};
    int last_error{};

    BufferSizeChecker buffer_size_checker;
    ZeroWriter zero_writer;
    Beep beeper;

    std::thread thread;
};

#endif //DWIPE_DISC_H
