//
// Created by bnozka on 30.05.2021.
//

#include <fcntl.h>
#include <unistd.h>

#include "BufferSizeChecker.h"

BufferSizeChecker::BufferSizeChecker(const std::filesystem::path& path, unsigned int& buffer_size):
path(path),
final_buffer_size(buffer_size)
{
}

void BufferSizeChecker::tick()
{
    if(tickable_state != RUNNING)
    {
        return;
    }

    switch (state) {
    case WRITING: {
        auto writen = write(file_descriptor, buffer, current_buffer_size);
        if (writen < 0)
        {
            last_error = errno;
            close(file_descriptor);
            tickable_state = ERROR;
            return;
        }
        current_writen += writen;

        auto duration{std::chrono::high_resolution_clock::now() - start};
        if (duration > writing_duration)
        {
            typeof(duration) converted_writing_duration{std::chrono::seconds {1}};;
            current_writen = (current_writen * converted_writing_duration.count())
                    / duration.count();
            if(current_writen > last_writen)
            {
                final_buffer_size = current_buffer_size;
                last_writen = current_writen;
            }
            close(file_descriptor);
            state = NEXT_BUFFER;
            return;
        }
        return;
    }
    case NEXT_BUFFER: {
        if (current_buffer_size >= max_buffer_size)
        {
            tickable_state = FINISH;
            state = INIT;
            return;
        }
        current_buffer_size <<= 1;
        current_writen = 0;
        file_descriptor = open(path.c_str(), O_WRONLY | O_DIRECT);
        start = std::chrono::high_resolution_clock::now();
        state = WRITING;
        return;
    }
    case INIT:
    {
        last_error = 0;
        last_writen = 0;
        // Size will be bumped up by NEXT_BUFFER step
        current_buffer_size = block_size/2;
        state = NEXT_BUFFER;
    }
    }
}
void BufferSizeChecker::reset()
{
    if(tickable_state != RUNNING)
    {
        state = INIT;
        TickableState::reset();
    }
}
void BufferSizeChecker::abort()
{
    if(tickable_state == RUNNING)
    {
        if(state == WRITING)
        {
            close(file_descriptor);
        }
        tickable_state = FINISH;
    }
}
