//
// Created by bnozka on 30.05.2021.
//

#include <fcntl.h>
#include <unistd.h>

#include "ZeroWriter.h"

ZeroWriter::ZeroWriter(const std::filesystem::path& path, const unsigned int& buffer_size, unsigned long size):
path(path),
buffer_size(buffer_size),
dev_size(size)
{
}

void ZeroWriter::tick()
{
    if(tickable_state != RUNNING)
    {
        return;
    }

    switch (state) {
    case INIT:
    {
        last_error = 0;
        current_writen = 0;
        file_descriptor = open(path.c_str(), O_WRONLY | O_DIRECT);
        start = std::chrono::high_resolution_clock::now();
        last_writen = 0;
        state = WRITING;
        break;
    }
    case WRITING:
    {
        auto writen = write(file_descriptor, buffer, buffer_size);
        if (writen < 0)
        {
            last_error = errno;
            close(file_descriptor);
            tickable_state = ERROR;
            return;
        }
        current_writen += writen;
        if(current_writen >= dev_size)
        {
            speed = 0;
            tickable_state = FINISH;
        }

        auto duration{std::chrono::high_resolution_clock::now() - start};
        if (duration > writing_duration)
        {
            typeof(duration) converted_writing_duration{std::chrono::seconds {1}};
            auto raw_speed = ((current_writen - last_writen) * converted_writing_duration.count()) / duration.count();
            speed = speed == 0 ? raw_speed : speed - (lpf_beta * (speed - raw_speed));
            last_writen = current_writen;
            start = std::chrono::high_resolution_clock::now();
        }
        break;
    }
    }
}
void ZeroWriter::reset()
{
    if(tickable_state != RUNNING)
    {
        state = INIT;
        TickableState::reset();
    }
}
void ZeroWriter::abort()
{
    if(tickable_state == RUNNING)
    {
        if(state == WRITING)
        {
            close(file_descriptor);
        }
        tickable_state = FINISH;
    }
}
